#
# random set cover problem
#

from coopr.pyomo import *
import math
import random

random.seed(9090)

model = Model()


#
# m - number of elements
#
model.m = Param(within=PositiveIntegers, default=10)
#
# n - number of sets
#
model.n = Param(within=PositiveIntegers, default=10)
#
# p - max number elements per set
#
model.p = Param(default=4, within=PositiveIntegers)

model.I = RangeSet(1,model.m)
model.J = RangeSet(1,model.n)

#
# = setof{i in J, j in 1 .. p} (i, ceil(Uniform(0,m)));
#
def S_rule(model):
    ans = set()
    for i in model.J:
        tmp = range(1,value(model.m)+1)
        random.shuffle( tmp )
        for j in range(0,value(model.p)):
            ans.add( (i, tmp[j]) )
    return ans

model.S = Set(dimen=2, initialize=S_rule)

#
# weights
#
model.w = Param(model.J, within=NonNegativeIntegers, initialize=1)

model.x = Var(model.J, within=Binary)


#
# Objective
#
def cost_rule(instance):
    return summation(instance.x, instance.w)

model.cost = Objective(rule=cost_rule)

#
# Constraint
#
def cover_rule(i, instance):
    expr = 0
    for j in instance.J:
        if (j,i) in instance.S:
            expr += instance.x[j]
    if expr is 0:
        return None
    return expr >= 1

model.cover = Constraint(model.I, rule=cover_rule)

